@@ -46,6 +46,9 @@
 #define FALSE   0
 #endif
 
+#ifndef tboolean
+#define tboolean   int
+#endif
 
 struct ipmi_intf;
 
@@ -60,7 +63,7 @@ struct oemvalstr {
 };
 
 const char * val2str(uint16_t val, const struct valstr * vs);
-const char * oemval2str(uint16_t oem,uint16_t val, const struct oemvalstr * vs);
+const char * oemval2str(uint32_t oem,uint16_t val, const struct oemvalstr * vs);
 uint16_t str2val(const char * str, const struct valstr * vs);
 void print_valstr(const struct valstr * vs, const char * title, int loglevel);
 void print_valstr_2col(const struct valstr * vs, const char * title, int loglevel);
