@@ -93,7 +93,7 @@ struct ipmi_session {
 	socklen_t addrlen;
 
 	/*
-	 * This struct holds state data specific to IMPI v2 / RMCP+ sessions
+	 * This struct holds state data specific to IPMI v2 / RMCP+ sessions
 	 */
 	struct {
 		enum LANPLUS_SESSION_STATE session_state;
@@ -173,6 +173,8 @@ struct ipmi_intf {
 	uint32_t target_addr;
 	uint8_t target_lun;
 	uint8_t target_channel;
+	uint32_t transit_addr;
+	uint8_t transit_channel;
 
 	uint8_t devnum;
 
