@@ -726,6 +726,7 @@ struct ipmi_sdr_iterator {
 	uint16_t reservation;
 	int total;
 	int next;
+	int use_built_in;
 };
 
 struct sdr_record_list {
@@ -791,7 +792,8 @@ static const char *sensor_type_desc[] __attribute__ ((unused)) = {
 	    "Management Subsystem Health", "Battery","Session Audit",
        "Version Change","FRU State" };
 
-struct ipmi_sdr_iterator *ipmi_sdr_start(struct ipmi_intf *intf);
+struct ipmi_sdr_iterator *ipmi_sdr_start(struct ipmi_intf *intf,
+                                         int use_builtin);
 struct sdr_get_rs *ipmi_sdr_get_next_header(struct ipmi_intf *intf,
 					    struct ipmi_sdr_iterator *i);
 uint8_t *ipmi_sdr_get_record(struct ipmi_intf *intf, struct sdr_get_rs *header,
@@ -820,11 +822,14 @@ struct ipmi_rs *ipmi_sdr_get_sensor_reading_ipmb(struct ipmi_intf *intf,
 						 uint8_t target,
 						 uint8_t lun);
 struct ipmi_rs *ipmi_sdr_get_sensor_thresholds(struct ipmi_intf *intf,
-					       uint8_t sensor);
+					       uint8_t sensor,
+					       uint8_t target, uint8_t lun);
 struct ipmi_rs *ipmi_sdr_get_sensor_hysteresis(struct ipmi_intf *intf,
-					       uint8_t sensor);
+					       uint8_t sensor,
+					       uint8_t target, uint8_t lun);
 const char *ipmi_sdr_get_sensor_type_desc(const uint8_t type);
-int ipmi_sdr_get_reservation(struct ipmi_intf *intf, uint16_t * reserve_id);
+int ipmi_sdr_get_reservation(struct ipmi_intf *intf, int use_builtin,
+                             uint16_t * reserve_id);
 
 int ipmi_sdr_print_sensor_full(struct ipmi_intf *intf,
 			       struct sdr_record_full_sensor *sensor);
@@ -845,7 +850,7 @@ int ipmi_sdr_print_sensor_entity_assoc(struct ipmi_intf *intf,
 struct sdr_record_list *ipmi_sdr_find_sdr_byentity(struct ipmi_intf *intf,
 						   struct entity_id *entity);
 struct sdr_record_list *ipmi_sdr_find_sdr_bynumtype(struct ipmi_intf *intf,
-						    uint8_t num, uint8_t type);
+					uint16_t gen_id, uint8_t num, uint8_t type);
 struct sdr_record_list *ipmi_sdr_find_sdr_bysensortype(struct ipmi_intf *intf,
 						       uint8_t type);
 struct sdr_record_list *ipmi_sdr_find_sdr_byid(struct ipmi_intf *intf,
@@ -864,9 +869,11 @@ void ipmi_sdr_print_discrete_state_mini(const char *separator,
 					uint8_t state1, uint8_t state2);
 int ipmi_sdr_print_sensor_event_status(struct ipmi_intf *intf,
 				       uint8_t sensor_num, uint8_t sensor_type,
-				       uint8_t event_type, int numeric_fmt);
+				       uint8_t event_type, int numeric_fmt,
+				       uint8_t target, uint8_t lun);
 int ipmi_sdr_print_sensor_event_enable(struct ipmi_intf *intf,
 				       uint8_t sensor_num, uint8_t sensor_type,
-				       uint8_t event_type, int numeric_fmt);
+				       uint8_t event_type, int numeric_fmt,
+				       uint8_t target, uint8_t lun);
 
 #endif				/* IPMI_SDR_H */
