@@ -28,10 +28,6 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
 #ifndef IPMI_SENSOR_H
@@ -84,5 +80,5 @@ struct sensor_set_thresh_rq {
 int ipmi_sensor_main(struct ipmi_intf *, int, char **);
 int ipmi_sensor_print_full(struct ipmi_intf *, struct sdr_record_full_sensor *);
 int ipmi_sensor_print_compact(struct ipmi_intf *, struct sdr_record_compact_sensor *);
-
+int ipmi_sensor_get_sensor_reading_factors( struct ipmi_intf * intf, struct sdr_record_full_sensor * sensor, uint8_t reading);
 #endif  /* IPMI_SENSOR_H */
