@@ -28,10 +28,6 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
 #ifndef IPMI_STRINGS_H
@@ -42,10 +38,10 @@
 extern const struct valstr completion_code_vals[];
 extern const struct valstr entity_id_vals[];
 extern const struct valstr entity_device_type_vals[];
-
+extern const struct valstr ipmi_netfn_vals[];
 extern const struct valstr ipmi_channel_activity_type_vals[];
 extern const struct valstr ipmi_privlvl_vals[];
-extern const struct valstr impi_bit_rate_vals[];
+extern const struct valstr ipmi_bit_rate_vals[];
 extern const struct valstr ipmi_set_in_progress_vals[];
 extern const struct valstr ipmi_authtype_session_vals[];
 extern const struct valstr ipmi_authtype_vals[];
@@ -55,6 +51,13 @@ extern const struct valstr ipmi_chassis_power_control_vals[];
 extern const struct valstr ipmi_auth_algorithms[];
 extern const struct valstr ipmi_integrity_algorithms[];
 extern const struct valstr ipmi_encryption_algorithms[];
+extern const struct valstr ipmi_oem_info[];
+
+/* these are similar, expect that the lookup takes the IANA number
+   as first parameter */
+extern const struct oemvalstr ipmi_oem_product_info[];
+extern const struct oemvalstr ipmi_oem_sdr_type_vals[];
+
 
 
 #endif /*IPMI_STRINGS_H*/
