@@ -119,12 +119,12 @@ const char * val2str(uint16_t val, const struct valstr *vs)
 	}
 
 	memset(un_str, 0, 32);
-	snprintf(un_str, 32, "Unknown (0x%x)", val);
+	snprintf(un_str, 32, "Unknown (0x%02X)", val);
 
 	return un_str;
 }
-
-const char * oemval2str(uint16_t oem, uint16_t val,
+ 
+const char * oemval2str(uint32_t oem, uint16_t val,
                                              const struct oemvalstr *vs)
 {
 	static char un_str[32];
@@ -148,7 +148,7 @@ const char * oemval2str(uint16_t oem, uint16_t val,
 	}
 
 	memset(un_str, 0, 32);
-	snprintf(un_str, 32, "OEM reserved #%02x", val);
+	snprintf(un_str, 32, "Unknown (0x%X)", val);
 
 	return un_str;
 }
