@@ -1256,8 +1256,8 @@ get_cmdline_ipaddr(char * arg, uint8_t * buf)
 
 static void ipmi_lan_set_usage(void)
 {
-	lprintf(LOG_NOTICE, "\nusage: lan set <channel> <command> [option]\n");
-	lprintf(LOG_NOTICE, "LAN set commands:");
+	lprintf(LOG_NOTICE, "\nusage: lan set <channel> <command> <parameter>\n");
+	lprintf(LOG_NOTICE, "LAN set command/parameter options:");
 	lprintf(LOG_NOTICE, "  ipaddr <x.x.x.x>               Set channel IP address");
 	lprintf(LOG_NOTICE, "  netmask <x.x.x.x>              Set channel IP netmask");
 	lprintf(LOG_NOTICE, "  macaddr <x:x:x:x:x:x>          Set channel MAC address");
@@ -1489,28 +1489,49 @@ ipmi_lan_set(struct ipmi_intf * intf, int argc, char ** argv)
 		}
 	}
 	/* ip address */
-	else if ((strncmp(argv[1], "ipaddr", 6) == 0) &&
-		 (get_cmdline_ipaddr(argv[2], data) == 0)) {
-		printf("Setting LAN %s to %d.%d.%d.%d\n",
-		       ipmi_lan_params[IPMI_LANP_IP_ADDR].desc,
-		       data[0], data[1], data[2], data[3]);
-		rc = set_lan_param(intf, chan, IPMI_LANP_IP_ADDR, data, 4);
+	else if (strncmp(argv[1], "ipaddr", 6) == 0) {
+		if(argc != 3)
+		{
+			ipmi_lan_set_usage();
+			return -1;
+		}
+		rc = get_cmdline_ipaddr(argv[2], data);
+		if (rc == 0) {
+			printf("Setting LAN %s to %d.%d.%d.%d\n",
+				ipmi_lan_params[IPMI_LANP_IP_ADDR].desc,
+				data[0], data[1], data[2], data[3]);
+			rc = set_lan_param(intf, chan, IPMI_LANP_IP_ADDR, data, 4);
+		}
 	}
 	/* network mask */
-	else if ((strncmp(argv[1], "netmask", 7) == 0) &&
-		 (get_cmdline_ipaddr(argv[2], data) == 0)) {
-		printf("Setting LAN %s to %d.%d.%d.%d\n",
-		       ipmi_lan_params[IPMI_LANP_SUBNET_MASK].desc,
-		       data[0], data[1], data[2], data[3]);
-		rc = set_lan_param(intf, chan, IPMI_LANP_SUBNET_MASK, data, 4);
+	else if (strncmp(argv[1], "netmask", 7) == 0) {
+		if(argc != 3)
+		{
+			ipmi_lan_set_usage();
+			return -1;
+		}
+		rc = get_cmdline_ipaddr(argv[2], data);
+		if (rc == 0) {
+			printf("Setting LAN %s to %d.%d.%d.%d\n",
+		       		ipmi_lan_params[IPMI_LANP_SUBNET_MASK].desc,
+		       		data[0], data[1], data[2], data[3]);
+			rc = set_lan_param(intf, chan, IPMI_LANP_SUBNET_MASK, data, 4);
+		}
 	}
 	/* mac address */
-	else if ((strncmp(argv[1], "macaddr", 7) == 0) &&
-		 (get_cmdline_macaddr(argv[2], data) == 0)) {
-		printf("Setting LAN %s to %02x:%02x:%02x:%02x:%02x:%02x\n",
-		       ipmi_lan_params[IPMI_LANP_MAC_ADDR].desc,
-		       data[0], data[1], data[2], data[3], data[4], data[5]);
-		rc = set_lan_param(intf, chan, IPMI_LANP_MAC_ADDR, data, 6);
+	else if (strncmp(argv[1], "macaddr", 7) == 0) {
+		if(argc != 3)
+		{
+			ipmi_lan_set_usage();
+			return -1;
+		}
+		rc = get_cmdline_macaddr(argv[2], data);
+		if (rc == 0) {
+			printf("Setting LAN %s to %02x:%02x:%02x:%02x:%02x:%02x\n",
+		       		ipmi_lan_params[IPMI_LANP_MAC_ADDR].desc,
+		       		data[0], data[1], data[2], data[3], data[4], data[5]);
+			rc = set_lan_param(intf, chan, IPMI_LANP_MAC_ADDR, data, 6);
+		}
 	}
 	/* default gateway settings */
 	else if (strncmp(argv[1], "defgw", 5) == 0) {
@@ -1531,6 +1552,10 @@ ipmi_lan_set(struct ipmi_intf * intf, int argc, char ** argv)
 			       data[0], data[1], data[2], data[3], data[4], data[5]);
 			rc = set_lan_param(intf, chan, IPMI_LANP_DEF_GATEWAY_MAC, data, 6);
 		}
+		else {
+			ipmi_lan_set_usage();
+			return -1;
+		}
 	}
 	/* backup gateway settings */
 	else if (strncmp(argv[1], "bakgw", 5) == 0) {
@@ -1551,6 +1576,10 @@ ipmi_lan_set(struct ipmi_intf * intf, int argc, char ** argv)
 			       data[0], data[1], data[2], data[3], data[4], data[5]);
 			rc = set_lan_param(intf, chan, IPMI_LANP_BAK_GATEWAY_MAC, data, 6);
 		}
+		else {
+			ipmi_lan_set_usage();
+			return -1;
+		}
 	}
 	else if (strncasecmp(argv[1], "vlan", 4) == 0) {
 		if (argc < 4 || strncmp(argv[2], "help", 4) == 0) {
@@ -1722,21 +1751,22 @@ ipmi_lan_alert_print_all(struct ipmi_intf * intf, uint8_t channel)
 static void
 ipmi_lan_alert_print_usage(void)
 {
-	lprintf(LOG_NOTICE, "\nusage: lan alert print [channel] [alert]\n");
+	lprintf(LOG_NOTICE, "\nusage: lan alert print [channel number] [alert destination]\n");
 	lprintf(LOG_NOTICE, "Default will print all alerts for the first found LAN channel");
 }
 
 static void
 ipmi_lan_alert_set_usage(void)
 {
-	lprintf(LOG_NOTICE, "\nusage: lan alert set <channel> <alert> <command> <option>\n");
-	lprintf(LOG_NOTICE, "  ipaddr <x.x.x.x>               Set alert IP address");
-	lprintf(LOG_NOTICE, "  macaddr <x:x:x:x:x:x>          Set alert MAC address");
-	lprintf(LOG_NOTICE, "  gateway <default|backup>       Set channel gateway to use for alerts");
-	lprintf(LOG_NOTICE, "  ack <on|off>                   Set Alert Acknowledge on or off");
-	lprintf(LOG_NOTICE, "  type <pet|oem1|oem2>           Set destination type as PET or OEM");
-	lprintf(LOG_NOTICE, "  time <seconds>                 Set ack timeout or unack retry interval");
-	lprintf(LOG_NOTICE, "  retry <number>                 Set number of alert retries");
+	lprintf(LOG_NOTICE, "\nusage: lan alert set <channel number> <alert destination> <command> <parameter>\n");
+	lprintf(LOG_NOTICE, "    Command/parameter options:\n");
+	lprintf(LOG_NOTICE, "    ipaddr <x.x.x.x>               Set alert IP address");
+	lprintf(LOG_NOTICE, "    macaddr <x:x:x:x:x:x>          Set alert MAC address");
+	lprintf(LOG_NOTICE, "    gateway <default|backup>       Set channel gateway to use for alerts");
+	lprintf(LOG_NOTICE, "    ack <on|off>                   Set Alert Acknowledge on or off");
+	lprintf(LOG_NOTICE, "    type <pet|oem1|oem2>           Set destination type as PET or OEM");
+	lprintf(LOG_NOTICE, "    time <seconds>                 Set ack timeout or unack retry interval");
+	lprintf(LOG_NOTICE, "    retry <number>                 Set number of alert retries");
 	lprintf(LOG_NOTICE, "");
 }
 
@@ -1994,12 +2024,12 @@ ipmi_lan_stats_get(struct ipmi_intf * intf, uint8_t chan)
 
 	rsp = intf->sendrecv(intf, &req);
 	if (rsp == NULL) {
-		lprintf(LOG_INFO, "Get LAN Stats command failed");
+		lprintf(LOG_ERR, "Get LAN Stats command failed");
 		return 0;
 	}
 
 	if (rsp->ccode > 0) {
-		lprintf(LOG_INFO, "Get LAN Stats command failed: %s",
+		lprintf(LOG_ERR, "Get LAN Stats command failed: %s",
 			val2str(rsp->ccode, completion_code_vals));
 		return 0;
 	}
@@ -2102,8 +2132,9 @@ print_lan_usage(void)
 {
 	lprintf(LOG_NOTICE, "LAN Commands:");
 	lprintf(LOG_NOTICE, "		   print [<channel number>]");
-	lprintf(LOG_NOTICE, "		   set [<channel number>]");
-	lprintf(LOG_NOTICE, "		   alert [<channel number>]");
+	lprintf(LOG_NOTICE, "		   set <channel number> <command> <parameter>");
+	lprintf(LOG_NOTICE, "		   alert print <channel number> <alert destination>");
+	lprintf(LOG_NOTICE, "		   alert set <channel number> <alert destination> <command> <parameter>");
 	lprintf(LOG_NOTICE, "		   stats get [<channel number>]");
 	lprintf(LOG_NOTICE, "		   stats clear [<channel number>]");
 }
