@@ -49,6 +49,7 @@
 #include <ipmitool/ipmi_intf.h>
 #include <ipmitool/ipmi_session.h>
 #include <ipmitool/ipmi_sdr.h>
+#include <ipmitool/ipmi_gendev.h>
 #include <ipmitool/ipmi_sel.h>
 #include <ipmitool/ipmi_fru.h>
 #include <ipmitool/ipmi_sol.h>
@@ -65,13 +66,14 @@
 #include <ipmitool/ipmi_raw.h>
 #include <ipmitool/ipmi_pef.h>
 #include <ipmitool/ipmi_oem.h>
+#include <ipmitool/ipmi_ekanalyzer.h>
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
 #ifdef ENABLE_ALL_OPTIONS
-# define OPTION_STRING	"I:hVvcgsEao:H:d:P:f:U:p:C:L:A:t:m:S:l:b:e:k:O:"
+# define OPTION_STRING	"I:hVvcgsEKao:H:d:P:f:U:p:C:L:A:t:T:m:S:l:b:B:e:k:y:O:"
 #else
 # define OPTION_STRING	"I:hVvcH:f:U:p:d:S:"
 #endif
@@ -228,15 +230,19 @@ ipmi_option_usage(const char * progname, struct ipmi_cmd * cmdlist, struct ipmi_
 	lprintf(LOG_NOTICE, "       -e char        Set SOL escape character");
 	lprintf(LOG_NOTICE, "       -C ciphersuite Cipher suite to be used by lanplus interface");
 	lprintf(LOG_NOTICE, "       -k key         Use Kg key for IPMIv2 authentication");
+	lprintf(LOG_NOTICE, "       -y hex_key     Use hexadecimal-encoded Kg key for IPMIv2 authentication");
 	lprintf(LOG_NOTICE, "       -L level       Remote session privilege level [default=ADMINISTRATOR]");
 	lprintf(LOG_NOTICE, "                      Append a '+' to use name/privilege lookup in RAKP1");
 	lprintf(LOG_NOTICE, "       -A authtype    Force use of auth type NONE, PASSWORD, MD2, MD5 or OEM");
 	lprintf(LOG_NOTICE, "       -P password    Remote session password");
 	lprintf(LOG_NOTICE, "       -E             Read password from IPMI_PASSWORD environment variable");
+	lprintf(LOG_NOTICE, "       -K             Read kgkey from IPMI_KGKEY environment variable");
 	lprintf(LOG_NOTICE, "       -m address     Set local IPMB address");
 	lprintf(LOG_NOTICE, "       -b channel     Set destination channel for bridged request");
-	lprintf(LOG_NOTICE, "       -l lun         Set destination lun for raw commands");
 	lprintf(LOG_NOTICE, "       -t address     Bridge request to remote target address");
+	lprintf(LOG_NOTICE, "       -B channel     Set transit channel for bridged request (dual bridge)");
+	lprintf(LOG_NOTICE, "       -T address     Set transit address for bridge request (dual bridge)");
+	lprintf(LOG_NOTICE, "       -l lun         Set destination lun for raw commands");
 	lprintf(LOG_NOTICE, "       -o oemtype     Setup for OEM (use 'list' to see available OEM types)");
 	lprintf(LOG_NOTICE, "       -O seloem      Use file for OEM SEL event descriptions");
 #endif
@@ -248,6 +254,67 @@ ipmi_option_usage(const char * progname, struct ipmi_cmd * cmdlist, struct ipmi_
 		ipmi_cmd_print(cmdlist);
 }
 
+/* ipmi_parse_hex - convert hexadecimal numbers to ascii string
+ *                  Input string must be composed of two-characer hexadecimal numbers.
+ *                  There is no separator between the numbers. Each number results in one character
+ *                  of the converted string.
+ *
+ *                  Example: ipmi_parse_hex("50415353574F5244") returns 'PASSWORD'
+ *
+ * @param str:  input string. It must contain only even number of '0'-'9','a'-'f' and 'A-F' characters.
+ * @returns converted ascii string
+ * @returns NULL on error
+ */
+static unsigned char *
+ipmi_parse_hex(const char *str)
+{
+	const char * p;
+	unsigned char * out, *q;
+	unsigned char b = 0;
+	int shift = 4;
+
+	if (strlen(str) == 0)
+		return NULL;
+
+	if (strlen(str) % 2 != 0) {
+		lprintf(LOG_ERR, "Number of hex_kg characters is not even");
+		return NULL;
+	}
+
+	if (strlen(str) > (IPMI_KG_BUFFER_SIZE-1)*2) {
+		lprintf(LOG_ERR, "Kg key is too long");
+		return NULL;
+	}
+
+	out = calloc(IPMI_KG_BUFFER_SIZE, sizeof(unsigned char));
+	if (out == NULL) {
+		lprintf(LOG_ERR, "malloc failure");
+		return NULL;
+	}
+
+	for (p = str, q = out; *p; p++) {
+		if (!isxdigit(*p)) {
+			lprintf(LOG_ERR, "Kg_hex is not hexadecimal number");
+			free(out);
+			return NULL;
+		}
+		
+		if (*p < 'A') /* it must be 0-9 */
+			b = *p - '0';
+		else /* it's A-F or a-f */
+			b = (*p | 0x20) - 'a' + 10; /* convert to lowercase and to 10-15 */
+
+		*q = *q + b << shift;
+		if (shift)
+			shift = 0;
+		else {
+			shift = 4;
+			q++;
+		}
+    }
+
+	return out;
+}
 
 /* ipmi_parse_options  -  helper function to handle parsing command line options
  *
@@ -269,6 +336,8 @@ ipmi_main(int argc, char ** argv,
 	int privlvl = 0;
 	uint8_t target_addr = 0;
 	uint8_t target_channel = 0;
+	uint8_t transit_addr = 0;
+	uint8_t transit_channel = 0;
 	uint8_t target_lun     = 0;
 	uint8_t my_addr = 0;
 	uint8_t lookupbit = 0x10;	/* use name-only lookup by default */
@@ -281,7 +350,7 @@ ipmi_main(int argc, char ** argv,
 	char * progname = NULL;
 	char * oemtype  = NULL;
 	char * sdrcache = NULL;
-	char * kgkey    = NULL;
+	unsigned char * kgkey = NULL;
 	char * seloem   = NULL;
 	int port = 0;
 	int devnum = 0;
@@ -380,6 +449,27 @@ ipmi_main(int argc, char ** argv,
 				goto out_free;
 			}
 			break;
+		case 'K':
+			if ((tmp = getenv("IPMI_KGKEY")))
+			{
+				if (kgkey)
+					free(kgkey);
+				kgkey = strdup(tmp);
+				if (kgkey == NULL) {
+					lprintf(LOG_ERR, "%s: malloc failure", progname);
+					goto out_free;
+				}
+			}
+			else {
+				lprintf(LOG_WARN, "Unable to read kgkey from environment");
+			}
+			break;
+		case 'y':
+			kgkey = ipmi_parse_hex(optarg);
+			if (kgkey == NULL) {
+				goto out_free;
+			}
+			break;
 		case 'U':
 			username = strdup(optarg);
 			if (username == NULL) {
@@ -473,6 +563,12 @@ ipmi_main(int argc, char ** argv,
 		case 'b':
 			target_channel = (uint8_t)strtol(optarg, NULL, 0);
 			break;
+		case 'T':
+			transit_addr = (uint8_t)strtol(optarg, NULL, 0);
+			break;
+		case 'B':
+			transit_channel = (uint8_t)strtol(optarg, NULL, 0);
+			break;
 		case 'l':
 			target_lun = (uint8_t)strtol(optarg, NULL, 0);
 			break;
@@ -596,6 +692,15 @@ ipmi_main(int argc, char ** argv,
 		if (intf->open != NULL)
 			intf->open(intf);
 		intf->target_addr = target_addr;
+
+      if (transit_addr > 0) {
+         intf->transit_addr    = transit_addr;
+         intf->transit_channel = transit_channel;
+      }
+      else
+      {
+         intf->transit_addr = intf->my_addr;
+      }
 		/* must be admin level to do this over lan */
 		ipmi_intf_session_set_privlvl(intf, IPMI_SESSION_PRIV_ADMIN);
 	}
