@@ -1,21 +1,21 @@
 /*
  * Copyright (c) 2003 Sun Microsystems, Inc.  All Rights Reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * Redistribution of source code must retain the above copyright
  * notice, this list of conditions and the following disclaimer.
- * 
+ *
  * Redistribution in binary form must reproduce the above copyright
  * notice, this list of conditions and the following disclaimer in the
  * documentation and/or other materials provided with the distribution.
- * 
+ *
  * Neither the name of Sun Microsystems, Inc. or the names of
  * contributors may be used to endorse or promote products derived
  * from this software without specific prior written permission.
- * 
+ *
  * This software is provided "AS IS," without a warranty of any kind.
  * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
  * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
@@ -155,7 +155,9 @@ printf_mc_usage(void)
 	struct bitfield_data * bf;
 	printf("MC Commands:\n");
 	printf("  reset <warm|cold>\n");
+	printf("  guid\n");
 	printf("  info\n");
+	printf("  watchdog <get|reset|off>\n");
 	printf("  selftest\n");
 	printf("  getenables\n");
 	printf("  setenables <option=on|off> ...\n");
@@ -165,6 +167,17 @@ printf_mc_usage(void)
 	}
 }
 
+
+static void
+print_watchdog_usage(void)
+{
+	lprintf(LOG_NOTICE, "usage: watchdog <command>:");
+	lprintf(LOG_NOTICE, "   get    :  Get Current Watchdog settings");
+	lprintf(LOG_NOTICE, "   reset  :  Restart Watchdog timer based on most recent settings");
+	lprintf(LOG_NOTICE, "   off    :  Shut off a running Watchdog timer");
+}
+
+
 /* ipmi_mc_get_enables  -  print out MC enables
  *
  * @intf:	ipmi inteface
@@ -248,12 +261,12 @@ ipmi_mc_set_enables(struct ipmi_intf * intf, int argc, char ** argv)
 			if (strncmp(argv[i], bf->name, nl) != 0)
 				continue;
 			if (strncmp(argv[i]+nl+1, "off", 3) == 0) {
-				printf("Disabling %s\n", bf->desc);
-				en &= ~bf->mask;
+					printf("Disabling %s\n", bf->desc);
+					en &= ~bf->mask;
 			}
 			else if (strncmp(argv[i]+nl+1, "on", 2) == 0) {
-				printf("Enabling %s\n", bf->desc);
-				en |= bf->mask;
+					printf("Enabling %s\n", bf->desc);
+					en |= bf->mask;
 			}
 			else {
 				lprintf(LOG_ERR, "Unrecognized option: %s", argv[i]);
@@ -278,7 +291,7 @@ ipmi_mc_set_enables(struct ipmi_intf * intf, int argc, char ** argv)
 	}
 	else if (rsp->ccode > 0) {
 		lprintf(LOG_ERR, "Set Global Enables command failed: %s",
-			val2str(rsp->ccode, completion_code_vals));
+		       val2str(rsp->ccode, completion_code_vals));
 		return -1;
 	}
 
@@ -314,6 +327,7 @@ ipmi_mc_get_deviceid(struct ipmi_intf * intf)
 	struct ipmi_rq req;
 	struct ipm_devid_rsp *devid;
 	int i;
+	const char *product=NULL;
 
 	memset(&req, 0, sizeof(req));
 	req.msg.netfn = IPMI_NETFN_APP;
@@ -332,7 +346,7 @@ ipmi_mc_get_deviceid(struct ipmi_intf * intf)
 	}
 
 	devid = (struct ipm_devid_rsp *) rsp->data;
-	printf("Device ID                 : %i\n", 
+	printf("Device ID                 : %i\n",
 		devid->device_id);
 	printf("Device Revision           : %i\n",
 		devid->device_revision & IPM_DEV_DEVICE_ID_REV_MASK);
@@ -343,7 +357,7 @@ ipmi_mc_get_deviceid(struct ipmi_intf * intf)
 		IPM_DEV_IPMI_VERSION_MAJOR(devid->ipmi_version),
 		IPM_DEV_IPMI_VERSION_MINOR(devid->ipmi_version));
 	printf("Manufacturer ID           : %lu\n",
-           (long)IPM_DEV_MANUFACTURER_ID(devid->manufacturer_id));
+		(long)IPM_DEV_MANUFACTURER_ID(devid->manufacturer_id));
    printf("Manufacturer Name         : %s\n",
             val2str( (long)IPM_DEV_MANUFACTURER_ID(devid->manufacturer_id), 
             ipmi_oem_info) );
@@ -351,8 +365,17 @@ ipmi_mc_get_deviceid(struct ipmi_intf * intf)
 	printf("Product ID                : %u (0x%02x%02x)\n",
 		buf2short((uint8_t *)(devid->product_id)),
 		devid->product_id[1], devid->product_id[0]);
+ 
+	product=oemval2str(IPM_DEV_MANUFACTURER_ID(devid->manufacturer_id),
+							 (devid->product_id[1]<<8)+devid->product_id[0],
+							 ipmi_oem_product_info);
+
+	if (product!=NULL) {
+		printf("Product Name              : %s\n", product);
+	}
+
 	printf("Device Available          : %s\n",
-		(devid->fw_rev1 & IPM_DEV_FWREV1_AVAIL_MASK) ? 
+		(devid->fw_rev1 & IPM_DEV_FWREV1_AVAIL_MASK) ?
 		"no" : "yes");
 	printf("Provides Device SDRs      : %s\n",
 		(devid->device_revision & IPM_DEV_DEVICE_ID_SDR_MASK) ?
@@ -418,13 +441,14 @@ ipmi_mc_get_guid(struct ipmi_intf * intf)
 		memset(&guid, 0, sizeof(struct ipmi_guid));
 		memcpy(&guid, rsp->data, rsp->data_len);
 
+		/* Kipp - changed order of last field (node) to follow specification */
 		printf("System GUID  : %08x-%04x-%04x-%04x-%02x%02x%02x%02x%02x%02x\n",
 		       guid.time_low, guid.time_mid, guid.time_hi_and_version,
 		       guid.clock_seq_hi_variant << 8 | guid.clock_seq_low,
-		       guid.node[5], guid.node[4], guid.node[3],
-		       guid.node[2], guid.node[1], guid.node[0]);
+		       guid.node[0], guid.node[1], guid.node[2],
+		       guid.node[3], guid.node[4], guid.node[5]);
 
-		s = (time_t)BSWAP_32(guid.time_low);
+		s = (time_t)guid.time_low; /* Kipp - removed the BSWAP_32, it was not needed here */
 		strftime(tbuf, sizeof(tbuf), "%m/%d/%Y %H:%M:%S", localtime(&s));
 		printf("Timestamp    : %s\n", tbuf);
 	}
@@ -456,9 +480,10 @@ static int ipmi_mc_get_selftest(struct ipmi_intf * intf)
 		lprintf(LOG_ERR, "No response from devices\n"); 
 		return -1;
 	}
+
 	if (rsp->ccode) {
 	   lprintf(LOG_ERR, "Bad response: (%s)",
-	                                 val2str(rsp->ccode, completion_code_vals));
+				val2str(rsp->ccode, completion_code_vals));
 		return -1;
 	}
 
@@ -503,15 +528,18 @@ static int ipmi_mc_get_selftest(struct ipmi_intf * intf)
 			printf("controller operational firmware corrupted\n");
 		}
 	}
+
 	else if (sft_res->code == IPM_SFT_CODE_FATAL_ERROR) {
 		printf("Selftest     : fatal error\n");
 		printf("Failure code : %02x\n", sft_res->test);
 		rv = -1;
 	}
+
 	else if (sft_res->code == IPM_SFT_CODE_RESERVED) {
 		printf("Selftest: N/A");
 		rv = -1;
 	}
+
 	else {
 		printf("Selttest     : device specific\n");
 		printf("Failure code : %02x\n", sft_res->test);
@@ -521,6 +549,171 @@ static int ipmi_mc_get_selftest(struct ipmi_intf * intf)
 	return rv;
 }
 
+/* ipmi_mc_get_watchdog
+ *
+ * @intf:	ipmi interface
+ *
+ * returns 0 on success
+ * returns -1 on error
+ */
+
+const char *wdt_use_string[8] = {
+	"Reserved",
+	"BIOS FRB2",
+	"BIOS/POST",
+	"OS Load",
+	"SMS/OS",
+	"OEM",
+	"Reserved",
+	"Reserved"
+};
+
+const char *wdt_action_string[8] = {
+	"No action",
+	"Hard Reset",
+	"Power Down",
+	"Power Cycle",
+	"Reserved",
+	"Reserved",
+	"Reserved",
+	"Reserved"
+};
+ 
+static int
+ipmi_mc_get_watchdog(struct ipmi_intf * intf)
+{
+	struct ipmi_rs * rsp;
+	struct ipmi_rq req;
+	struct ipm_get_watchdog_rsp * wdt_res;
+	
+	memset(&req, 0, sizeof(req));
+	req.msg.netfn = IPMI_NETFN_APP;
+	req.msg.cmd = BMC_GET_WATCHDOG_TIMER;
+	req.msg.data_len = 0;
+
+	rsp = intf->sendrecv(intf, &req);
+	if (rsp == NULL) {
+		lprintf(LOG_ERR, "Get Watchdog Timer command failed");
+		return -1;
+	}
+
+	if (rsp->ccode) {
+		lprintf(LOG_ERR, "Get Watchdog Timer command failed: %s",
+			val2str(rsp->ccode, completion_code_vals));
+		return -1;
+	}
+	
+	wdt_res = (struct ipm_get_watchdog_rsp *) rsp->data;
+	
+	printf("Watchdog Timer Use:     %s (0x%02x)\n", 
+      wdt_use_string[(wdt_res->timer_use & 0x07 )], wdt_res->timer_use);
+	printf("Watchdog Timer Is:      %s\n", 
+		wdt_res->timer_use & 0x40 ? "Started/Running" : "Stopped");
+	printf("Watchdog Timer Actions: %s (0x%02x)\n", 
+		 wdt_action_string[(wdt_res->timer_actions&0x07)], wdt_res->timer_actions);
+	printf("Pre-timeout interval:   %d seconds\n", wdt_res->pre_timeout);
+	printf("Timer Expiration Flags: 0x%02x\n", wdt_res->timer_use_exp);
+	printf("Initial Countdown:      %i sec\n", 
+	    ((wdt_res->initial_countdown_msb << 8) | wdt_res->initial_countdown_lsb)/10 );
+	printf("Present Countdown:      %i sec\n", 
+	    (((wdt_res->present_countdown_msb << 8) | wdt_res->present_countdown_lsb)) / 10);
+	
+	
+	return 0;	
+}
+
+/* ipmi_mc_shutoff_watchdog
+ *
+ * @intf:	ipmi interface
+ *
+ * returns 0 on success
+ * returns -1 on error
+ */
+static int
+ipmi_mc_shutoff_watchdog(struct ipmi_intf * intf)
+{
+	struct ipmi_rs * rsp;
+	struct ipmi_rq req;
+	unsigned char msg_data[6];
+	
+	memset(&req, 0, sizeof(req));
+	req.msg.netfn = IPMI_NETFN_APP;
+	req.msg.cmd   = BMC_SET_WATCHDOG_TIMER;
+	req.msg.data  = msg_data;
+	req.msg.data_len = 6;
+
+	/*
+	 * The only set cmd we're allowing is to shut off the timer.
+	 * Turning on the timer should be the job of the ipmi watchdog driver.
+	 * See 'modinfo ipmi_watchdog' for more info. (NOTE: the reset 
+	 * command will restart the timer if it's already been initialized.)
+	 *
+	 * Out-of-band watchdog set commands can still be sent via the raw
+	 * command interface but this is a very dangerous thing to do since
+	 * a periodic "poke"/reset over a network is unreliable.  This is
+	 * not a recommended way to use the IPMI watchdog commands. 
+	 */
+	
+	msg_data[0] = IPM_WATCHDOG_SMS_OS;
+	msg_data[1] = IPM_WATCHDOG_NO_ACTION;
+	msg_data[2] = 0x00;  // pretimeout interval
+	msg_data[3] = IPM_WATCHDOG_CLEAR_SMS_OS;
+	msg_data[4] = 0xb8;  // countdown lsb (100 ms/count)
+	msg_data[5] = 0x0b;  // countdown msb - 5 mins
+
+	rsp = intf->sendrecv(intf, &req);
+	if (rsp == NULL) {
+		lprintf(LOG_ERR, "Watchdog Timer Shutoff command failed!");
+		return -1;
+	}
+
+	if (rsp->ccode) {
+		lprintf(LOG_ERR, "Watchdog Timer Shutoff command failed! %s",
+			val2str(rsp->ccode, completion_code_vals));
+		return -1;
+	}
+		
+	lprintf(LOG_ERR, "Watchdog Timer Shutoff successful -- timer stopped");
+	return 0;	
+}
+
+
+/* ipmi_mc_rst_watchdog
+ *
+ * @intf:	ipmi interface
+ *
+ * returns 0 on success
+ * returns -1 on error
+ */
+static int
+ipmi_mc_rst_watchdog(struct ipmi_intf * intf)
+{
+	struct ipmi_rs * rsp;
+	struct ipmi_rq req;
+	
+	memset(&req, 0, sizeof(req));
+	req.msg.netfn = IPMI_NETFN_APP;
+	req.msg.cmd   = BMC_RESET_WATCHDOG_TIMER;
+	req.msg.data_len = 0;
+
+	rsp = intf->sendrecv(intf, &req);
+	if (rsp == NULL) {
+		lprintf(LOG_ERR, "Reset Watchdog Timer command failed!");
+		return -1;
+	}
+
+	if (rsp->ccode) {
+		lprintf(LOG_ERR, "Reset Watchdog Timer command failed: %s",
+			(rsp->ccode == IPM_WATCHDOG_RESET_ERROR) ? 
+				"Attempt to reset unitialized watchdog" :
+				val2str(rsp->ccode, completion_code_vals));
+		return -1;
+	}
+		
+	lprintf(LOG_ERR, "IPMI Watchdog Timer Reset -  countdown restarted!");
+	return 0;	
+}
+
 /* ipmi_mc_main  -  top-level handler for MC functions
  *
  * @intf:	ipmi interface
@@ -534,7 +727,7 @@ int
 ipmi_mc_main(struct ipmi_intf * intf, int argc, char ** argv)
 {
 	int rc = 0;
-
+	
 	if (argc < 1 || strncmp(argv[0], "help", 4) == 0) {
 		printf_mc_usage();
 	}
@@ -554,23 +747,39 @@ ipmi_mc_main(struct ipmi_intf * intf, int argc, char ** argv)
 	}
 	else if (strncmp(argv[0], "info", 4) == 0) {
 		rc = ipmi_mc_get_deviceid(intf);
-	}
+		}
 	else if (strncmp(argv[0], "guid", 4) == 0) {
 		rc = ipmi_mc_get_guid(intf);
-	}
+		}
 	else if (strncmp(argv[0], "getenables", 10) == 0) {
 		rc = ipmi_mc_get_enables(intf);
 	}
 	else if (strncmp(argv[0], "setenables", 10) == 0) {
 		rc = ipmi_mc_set_enables(intf, argc-1, &(argv[1]));
 	}
-    else if (!strncmp(argv[0], "selftest", 8)) {
+	else if (!strncmp(argv[0], "selftest", 8)) {
 		rc = ipmi_mc_get_selftest(intf);
 	}
-    else {
+	else if (!strncmp(argv[0], "watchdog", 8)) {
+		if (argc < 2 || strncmp(argv[1], "help", 4) == 0) {
+			print_watchdog_usage(); 
+		}
+		else if (strncmp(argv[1], "get", 3) == 0) {
+			rc = ipmi_mc_get_watchdog(intf);
+		}
+		else if(strncmp(argv[1], "off", 3) == 0) {
+			rc = ipmi_mc_shutoff_watchdog(intf);
+		}
+		else if(strncmp(argv[1], "reset", 5) == 0) {
+			rc = ipmi_mc_rst_watchdog(intf);
+		}
+		else {
+			print_watchdog_usage(); 
+		}
+	}
+	else {
 		lprintf(LOG_ERR, "Invalid mc/bmc command: %s", argv[0]);
 		rc = -1;
 	}
-
 	return rc;
 }
