@@ -28,10 +28,6 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
 #include <stdlib.h>
@@ -97,19 +93,11 @@ print_session_info_csv(const struct  get_session_info_rsp * session_info,
 	if (data_len == 18)
 	{
 		/* We have 802.3 LAN data */
-#ifdef __CYGWIN__
-		printf(",%d.%d.%d.%d",
-			session_info->channel_data.lan_data.console_ip[0],
-			session_info->channel_data.lan_data.console_ip[1],
-			session_info->channel_data.lan_data.console_ip[2],
-			session_info->channel_data.lan_data.console_ip[3]);
-#else
 		printf(",%s",
 			   inet_ntop(AF_INET,
 						 &(session_info->channel_data.lan_data.console_ip),
 						 buffer,
 						 16));
-#endif
 
 		printf(",%02x:%02x:%02x:%02x:%02x:%02x",
 			   session_info->channel_data.lan_data.console_mac[0],
@@ -137,19 +125,11 @@ print_session_info_csv(const struct  get_session_info_rsp * session_info,
 		printf(",%d",
 			   session_info->channel_data.modem_data.destination_selector);
 
-#ifdef __CYGWIN__
-		printf(",%d.%d.%d.%d",
-			session_info->channel_data.modem_data.console_ip[0],
-			session_info->channel_data.modem_data.console_ip[1],
-			session_info->channel_data.modem_data.console_ip[2],
-			session_info->channel_data.modem_data.console_ip[3]);
-#else
 		printf(",%s",
 			   inet_ntop(AF_INET,
 						 &(session_info->channel_data.modem_data.console_ip),
 						 buffer,
 						 16));
-#endif
 
 		if (data_len == 14)
 		{
@@ -201,19 +181,11 @@ print_session_info_verbose(const struct  get_session_info_rsp * session_info,
 	if (data_len == 18)
 	{
 		/* We have 802.3 LAN data */
-#ifdef __CYGWIN__
-		printf("console ip                    : %d.%d.%d.%d\n",
-			session_info->channel_data.lan_data.console_ip[0],
-			session_info->channel_data.lan_data.console_ip[1],
-			session_info->channel_data.lan_data.console_ip[2],
-			session_info->channel_data.lan_data.console_ip[3]);
-#else
 		printf("console ip                    : %s\n",
 			   inet_ntop(AF_INET,
 						 &(session_info->channel_data.lan_data.console_ip),
 						 buffer,
 						 16));
-#endif
 
 		printf("console mac                   : %02x:%02x:%02x:%02x:%02x:%02x\n",
 			   session_info->channel_data.lan_data.console_mac[0],
@@ -241,19 +213,11 @@ print_session_info_verbose(const struct  get_session_info_rsp * session_info,
 		printf("Destination selector          : %d\n",
 			   session_info->channel_data.modem_data.destination_selector);
 
-#ifdef __CYGWIN__
-		printf("console ip                    : %d.%d.%d.%d\n",
-			session_info->channel_data.modem_data.console_ip[0],
-			session_info->channel_data.modem_data.console_ip[1],
-			session_info->channel_data.modem_data.console_ip[2],
-			session_info->channel_data.modem_data.console_ip[3]);
-#else
 		printf("console ip                    : %s\n",
 			   inet_ntop(AF_INET,
 						 &(session_info->channel_data.modem_data.console_ip),
 						 buffer,
 						 16));
-#endif
 
 		if (data_len == 14)
 		{
