@@ -71,6 +71,8 @@
 #define SOL_PARAMETER_SOL_PAYLOAD_CHANNEL       0x07
 #define SOL_PARAMETER_SOL_PAYLOAD_PORT          0x08
 
+#define MAX_SOL_RETRY           		6
+
 const struct valstr sol_parameter_vals[] = {
 	{ SOL_PARAMETER_SET_IN_PROGRESS,           "Set In Progress (0)" },
 	{ SOL_PARAMETER_SOL_ENABLE,                "Enable (1)" },
@@ -90,6 +92,7 @@ static struct termios _saved_tio;
 static int            _in_raw_mode = 0;
 static int            _disable_keepalive = 0;
 static int            _use_sol_for_keepalive = 0;
+static int            _keepalive_retries = 0;
 
 extern int verbose;
 
@@ -140,6 +143,51 @@ ipmi_sol_payload_access(struct ipmi_intf * intf,
 	return -1;
 }
 
+int
+ipmi_sol_payload_access_status(struct ipmi_intf * intf,
+				uint8_t channel,
+				uint8_t userid)
+{
+	struct ipmi_rq req;
+	struct ipmi_rs *rsp;
+	uint8_t data[2];
+
+	memset(&req, 0, sizeof(req));
+	req.msg.netfn    = IPMI_NETFN_APP;
+	req.msg.cmd      = IPMI_GET_USER_PAYLOAD_ACCESS;
+	req.msg.data     = data;
+	req.msg.data_len = sizeof(data);
+
+	data[0] = channel & 0xf;	/* channel */
+	data[1] = userid & 0x3f;	/* user id */
+	rsp = intf->sendrecv(intf, &req);
+
+	if (rsp == NULL) {
+		lprintf(LOG_ERR, "Error: Unexpected data length (%d) received",
+			rsp->data_len);
+		return -1;
+	}
+
+	switch(rsp->ccode) {
+		case 0x00:
+			if (rsp->data_len != 4) {
+				lprintf(LOG_ERR, "Error parsing SOL payload status for user %d on channel %d",
+					userid, channel);
+				return -1;
+			}
+
+			printf("User %d on channel %d is %sabled\n",
+				userid, channel, (rsp->data[0] & 0x02) ? "en":"dis");
+			return 0;
+
+		default:
+			lprintf(LOG_ERR, "Error getting SOL payload status for user %d on channel %d: %s",
+				userid, channel, 
+				val2str(rsp->ccode, completion_code_vals));
+			return -1;
+	}
+}
+
 
 /*
  * ipmi_get_sol_info
@@ -156,7 +204,7 @@ ipmi_get_sol_info(
 
 	memset(&req, 0, sizeof(req));
 	req.msg.netfn    = IPMI_NETFN_TRANSPORT;
-	req.msg.cmd      = IMPI_GET_SOL_CONFIG_PARAMETERS;
+	req.msg.cmd      = IPMI_GET_SOL_CONFIG_PARAMETERS;
 	req.msg.data_len = 4;
 	req.msg.data     = data;
 
@@ -487,9 +535,13 @@ ipmi_get_sol_info(
 				}
 				break;
 			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported - defaulting to %d",
-				    val2str(data[1], sol_parameter_vals), intf->session->port);
-				params->payload_port = intf->session->port;
+            if( intf->session != NULL ){
+               lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported - defaulting to %d", val2str(data[1], sol_parameter_vals), intf->session->port);
+               params->payload_port = intf->session->port;
+            } else {
+               lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported - can't determine which payload port to use on NULL session" );
+               return -1;               
+            }
 				break;
 			default:
 				lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
@@ -533,10 +585,10 @@ ipmi_print_sol_info(struct ipmi_intf * intf, uint8_t channel)
 		printf("%d,", params.retry_interval * 10);
 
 		printf("%s,",
-			   val2str(params.volatile_bit_rate, impi_bit_rate_vals));
+			   val2str(params.volatile_bit_rate, ipmi_bit_rate_vals));
 
 		printf("%s,",
-			   val2str(params.non_volatile_bit_rate, impi_bit_rate_vals));
+			   val2str(params.non_volatile_bit_rate, ipmi_bit_rate_vals));
 
 		printf("%d,", params.payload_channel);
 		printf("%d\n", params.payload_port);
@@ -564,10 +616,10 @@ ipmi_print_sol_info(struct ipmi_intf * intf, uint8_t channel)
 			   params.retry_interval * 10);
 
 		printf("Volatile Bit Rate (kbps)        : %s\n",
-			   val2str(params.volatile_bit_rate, impi_bit_rate_vals));
+			   val2str(params.volatile_bit_rate, ipmi_bit_rate_vals));
 
 		printf("Non-Volatile Bit Rate (kbps)    : %s\n",
-			   val2str(params.non_volatile_bit_rate, impi_bit_rate_vals));
+			   val2str(params.non_volatile_bit_rate, ipmi_bit_rate_vals));
 
 		printf("Payload Channel                 : %d (0x%02x)\n",
 			   params.payload_channel, params.payload_channel);
@@ -580,6 +632,42 @@ ipmi_print_sol_info(struct ipmi_intf * intf, uint8_t channel)
 
 
 
+/*
+ * Small function to validate that user-supplied SOL
+ * configuration parameter values we store in uint8_t
+ * data type falls within valid range.  With minval
+ * and maxval parameters we can use the same function
+ * to validate parameters that have different ranges
+ * of values.
+ *
+ * function will return -1 if value is not valid, or
+ * will return 0 if valid.
+ */
+int ipmi_sol_set_param_isvalid_uint8_t( const char *strval,
+					const char *name,
+					int base,
+					uint8_t minval,
+					uint8_t maxval,
+					uint8_t *out_value)
+{
+	char *end;
+	long val = strtol(strval, &end, base);
+
+	if ((val < minval)
+			|| (val > maxval)
+			|| (*end != '\0')) {
+		lprintf(LOG_ERR, "Invalid value %s for parameter %s",
+			strval, name);
+		lprintf(LOG_ERR, "Valid values are %d-%d", minval, maxval);
+		return -1;
+	}
+	else {
+		*out_value = val;
+		return 0;
+	}
+}
+
+
 /*
  * ipmi_sol_set_param
  *
@@ -603,7 +691,7 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 
 	memset(&req, 0, sizeof(req));
 	req.msg.netfn    = IPMI_NETFN_TRANSPORT;           /* 0x0c */
-	req.msg.cmd      = IMPI_SET_SOL_CONFIG_PARAMETERS; /* 0x21 */
+	req.msg.cmd      = IPMI_SET_SOL_CONFIG_PARAMETERS; /* 0x21 */
 	req.msg.data     = data;
 
 	data[0] = channel;
@@ -777,7 +865,10 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 
 		req.msg.data_len = 4;
 		data[1] = SOL_PARAMETER_CHARACTER_INTERVAL;
-		data[2] = (uint8_t)strtol(value, NULL, 0);
+
+		/* validate user-supplied input */
+		if (ipmi_sol_set_param_isvalid_uint8_t(value, param, 0, 1, 255, &data[2]))
+			return -1;
 
 		/* We need other values to complete the request */
 		if (ipmi_get_sol_info(intf, channel, &params))
@@ -800,7 +891,10 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 
 		req.msg.data_len = 4;
 		data[1] = SOL_PARAMETER_CHARACTER_INTERVAL;
-		data[3] = (uint8_t)strtol(value, NULL, 0);
+
+		/* validate user-supplied input */
+		if (ipmi_sol_set_param_isvalid_uint8_t(value, param, 0, 0, 255, &data[3]))
+			return -1;
 
 		/* We need other values to complete the request */
 		if (ipmi_get_sol_info(intf, channel, &params))
@@ -823,7 +917,10 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 
 		req.msg.data_len = 4;
 		data[1] = SOL_PARAMETER_SOL_RETRY;
-		data[2] = (uint8_t)strtol(value, NULL, 0) & 0x07;
+
+		/* validate user input, 7 is max value */
+		if (ipmi_sol_set_param_isvalid_uint8_t(value, param, 0, 0, 7, &data[2]))
+			return -1;
 
 		/* We need other values to complete the request */
 		if (ipmi_get_sol_info(intf, channel, &params))
@@ -846,7 +943,10 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 
 		req.msg.data_len = 4;
 		data[1] = SOL_PARAMETER_SOL_RETRY;
-		data[3] = (uint8_t)strtol(value, NULL, 0);
+
+		/* validate user-supplied input */
+		if (ipmi_sol_set_param_isvalid_uint8_t(value, param, 0, 0, 255, &data[3]))
+			return -1;
 
 		/* We need other values to complete the request */
 		if (ipmi_get_sol_info(intf, channel, &params))
@@ -1180,7 +1280,7 @@ output(struct ipmi_rs * rsp)
 
 
 /*
- * impi_sol_deactivate
+ * ipmi_sol_deactivate
  */
 static int
 ipmi_sol_deactivate(struct ipmi_intf * intf)
@@ -1407,6 +1507,12 @@ ipmi_sol_keepalive_using_getdeviceid(struct ipmi_intf * intf)
 
 	if (end.tv_sec - _start_keepalive.tv_sec > SOL_KEEPALIVE_TIMEOUT) {
 	   ret = intf->keepalive(intf);
+	   if ( (ret!=0) && (_keepalive_retries < SOL_KEEPALIVE_RETRIES) ) {
+         ret = 0;
+         _keepalive_retries++;
+	   }
+	   else if ((ret==0) && (_keepalive_retries > 0))
+         _keepalive_retries = 0;
 		gettimeofday(&_start_keepalive, 0);
    }
 	return ret;
@@ -1414,8 +1520,6 @@ ipmi_sol_keepalive_using_getdeviceid(struct ipmi_intf * intf)
 
 
 
-
-
 /*
  * ipmi_sol_red_pill
  */
@@ -1431,6 +1535,7 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 	int    retval;
 	int    buffer_size = intf->session->sol_data.max_inbound_payload_size;
 	int    keepAliveRet = 0;
+	int    retrySol = 0;
 
 	buffer = (char*)malloc(buffer_size);
 	if (buffer == NULL) {
@@ -1450,20 +1555,40 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 		FD_SET(intf->fd, &read_fds);
 
 		/* Send periodic keepalive packet */
-      if(_use_sol_for_keepalive == 0)
-      {
-         keepAliveRet = ipmi_sol_keepalive_using_getdeviceid(intf);
-      }
-      else
-      {
-         keepAliveRet = ipmi_sol_keepalive_using_sol(intf);
-      }
+		if(_use_sol_for_keepalive == 0)
+		{
+			keepAliveRet = ipmi_sol_keepalive_using_getdeviceid(intf);
+		}
+		else
+		{
+			keepAliveRet = ipmi_sol_keepalive_using_sol(intf);
+		}
 		
 		if (keepAliveRet != 0)
 		{
-			/* no response to keepalive message */
-			bShouldExit = 1;
-			continue;
+			/*
+			 * Retrying the keep Alive before declaring a communication
+			 * lost state with the IPMC. Helpful when the payload is 
+			 * reset and brings down the connection temporarily. Otherwise,
+			 * if we send getDevice Id to check the status of IPMC during
+			 * this down time when the connection is restarting, SOL will 
+			 * exit even though the IPMC is available and the session is open.
+			 */
+			if (retrySol == MAX_SOL_RETRY)
+			{
+				/* no response to Get Device ID keepalive message */
+				bShouldExit = 1;
+				continue;
+			}
+			else 
+			{ 
+				retrySol++;         
+			}
+		}
+		else
+		{
+			/* if the keep Alive is successful reset retries to zero */
+			retrySol = 0;
 		}
 
 		/* Wait up to half a second */
@@ -1564,7 +1689,7 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 
 
 /*
- * impi_sol_activate
+ * ipmi_sol_activate
  */
 static int
 ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
@@ -1739,8 +1864,8 @@ print_sol_usage(void)
 {
 	lprintf(LOG_NOTICE, "SOL Commands: info [<channel number>]");
 	lprintf(LOG_NOTICE, "              set <parameter> <value> [channel]");
-	lprintf(LOG_NOTICE, "              payload <enable|disable> [channel] [userid]");
-	lprintf(LOG_NOTICE, "              activate [<usesolforkeepalive|nokeepalive>]");
+	lprintf(LOG_NOTICE, "              payload <enable|disable|status> [channel] [userid]");
+	lprintf(LOG_NOTICE, "              activate [<usesolkeepalive|nokeepalive>]");
 	lprintf(LOG_NOTICE, "              deactivate");
 	lprintf(LOG_NOTICE, "              looptest [<loop times>] [<loop interval(in ms)>]");
 }
@@ -1814,15 +1939,13 @@ ipmi_sol_main(struct ipmi_intf * intf, int argc, char ** argv)
 		uint8_t userid = 1;
 		int enable = -1;
 
-		if (!strncmp(argv[1], "enable", 6))
+		if (argc == 1 || argc > 4)
 		{
-			enable = 1;
+			print_sol_usage();
+			return -1;
 		}
-		else if (!strncmp(argv[1], "disable", 7))
-		{
-			enable = 0;
-		}
-		else
+
+		if (argc == 1 || argc > 4)
 		{
 			print_sol_usage();
 			return -1;
@@ -1832,11 +1955,29 @@ ipmi_sol_main(struct ipmi_intf * intf, int argc, char ** argv)
 		{
 			channel = (uint8_t)strtol(argv[2], NULL, 0);
 		}
-		if (argc >= 4)
+		if (argc == 4)
 		{
 			userid = (uint8_t)strtol(argv[3], NULL, 0);
 		}
 
+		if (!strncmp(argv[1], "enable", 6))
+		{
+			enable = 1;
+		}
+		else if (!strncmp(argv[1], "disable", 7))
+		{
+			enable = 0;
+		}
+		else if (!strncmp(argv[1], "status", 6))
+		{
+			return ipmi_sol_payload_access_status(intf, channel, userid);
+		}
+		else
+		{
+			print_sol_usage();
+			return -1;
+		}
+
 		retval = ipmi_sol_payload_access(intf, channel, userid, enable);
 	}
 
@@ -1880,13 +2021,21 @@ ipmi_sol_main(struct ipmi_intf * intf, int argc, char ** argv)
 	 */
  	else if (!strncmp(argv[0], "activate", 8)) {
 
-		if (argc > 1) {
-			if (!strncmp(argv[1], "usesolkeepalive", 11))
+		if (argc > 2) {
+			print_sol_usage();
+			return -1;
+		}
+
+		if (argc == 2) {
+			if (!strncmp(argv[1], "usesolkeepalive", 15))
 				_use_sol_for_keepalive = 1;
 			else if (!strncmp(argv[1], "nokeepalive", 11))
 				_disable_keepalive = 1;
+			else {
+				print_sol_usage();
+				return -1;
+			}
 		}
-
 		retval = ipmi_sol_activate(intf, 0, 0);
 	}
 
